-- Keys are custom-style names.
-- Values are markup to place prefix/postfix content with
-- and optional separator to join div markup with with
-- (the default join is "\n", included here for illustration)
local style2latex = {
  Div = {
    -- Centered = {
    --   pre = '\\begin{center}',
    --   post = '\\end{center}',
    --   join = "\n"
    -- }
  },
  Span = {
    -- Red = {
    --   pre = '\\textcolor[HTML]{FF0000}{',
    --   post = '}'
    -- }
  }
}

-- A shim between error() and string.format()/sprintf
local function errorf (...)
  local msg = string.format(...)
  return error(msg, 2)
end

local please_upgrade = "Please upgrade <https://github.com/jgm/pandoc/releases/>"
ptp = pandoc.types or errorf("Cannot get the pandoc.types library. %s", please_upgrade)

if (not PANDOC_VERSION) or ptp.Version('3.1.4') > PANDOC_VERSION then
  return errorf("Your version of pandoc is too old for this filter. %s", please_upgrade)
end

local pdc = assert(pandoc, "Cannot find the pandoc library")
if not ('table' == type(pdc)) then
  error("Expected variable pandoc to be table")
end

local pdu = assert(pandoc.utils, "Cannot find the pandoc.utils library")
local pth = assert(pandoc.path, "Cannot get the pandoc.path library")
local jsn = assert(pandoc.json, "Cannot get the pandoc.json library")

-- Setup for converting content to LaTeX
local flavor = {
  format = 'latex',
  extensions = PANDOC_WRITER_OPTIONS.extensions
}

-- Convert content to LaTeX
local function to_latex (got, strip)
  if not ('Pandoc' == got.tag) then
    local arg_type = pdu.type(got)
    if not ('Blocks' == arg_type) then
      if 'Block' == arg_type then
        got = { got }
      elseif 'Inlines' == arg_type then
        got = { pdc.Plain(got) }
      elseif 'Inline' == arg_type then
        got = { pdc.Plain({ got }) }
      else
        errorf('Cannot convert %s to blocks', arg_type)
      end
    end
    got = pdc.Pandoc(got)
  end
  local latex = pdc.write(got, flavor, PANDOC_WRITER_OPTIONS)
  if strip then
    latex = latex:gsub("^%s+", ""):gsub("%s+$", "")
  end
  return latex
end

local read_file
do
  -- Do simple expansion of string-initial characters:
  -- `~` is replaced with the value of the HOME env variable
  -- `*` is replaced with the path to the user's Pandoc data directory
  local pat = '^[%~%*]'
  local subst = {
    ['~'] = os.getenv('HOME'),
    ['*'] = PANDOC_STATE.user_data_dir
  }
  read_file = function(fn, id)
    if id == nil then
      id = fn
    end
    local path = pth.normalize(fn:gsub(pat, subst))
    local fh, err = io.open(path, 'r')
    if not (fh) then
      errorf("Couldn't open %s (%s) for reading: %s", path, id, err)
    end
    local str
    str, err = fh:read('a')
    if not (str) then
      errorf("Couldn't read from %s (%s): %s", path, id, err)
    end
    fh:close()
    return str
  end
end

-- Load JSON (an object) from a file
local function load_json (fn, id)
  if id == nil then
    id = fn
  end
  local json = read_file(fn, id)
  local ok, got = pcall(jsn.decode, json, false)
  if not (ok) then
    errorf("Couldn't decode JSON from %s: %s", id, got)
  end
  if not ('table' == type(got)) then
    errorf("Expected %s to contain object", id)
  end
  return got
end

-- Curry raw constructors
local raw = {
  Block = function(s)
    return pdc.RawBlock('latex', s)
  end,
  Inline = function(s)
    return pdc.RawInline('latex', s)
  end
}

-- Turn a string value into a wrap definition
-- assuming the string is an environment or command name...
local wrap_str = {
  Div = {
    pre = function(s)
      return string.format('\\begin{%s}', s)
    end,
    post = function(s)
      return string.format('\\end{%s}', s)
    end
  },
  Span = {
    pre = function(s)
      return string.format('\\%s{', s)
    end,
    post = function()
      return '}'
    end
  }
}
-- ...using this function
local function str_wrap (s, tag)
  local wrap = { }
  for k, d in pairs(wrap_str[tag]) do
    wrap[k] = d(s)
  end
  return wrap
end

-- Definition keys which must have false or string values
local wrap_strings = {
  'pre',
  'post',
  'join'
}

local wrap_tags = {
  'Div',
  'Span'
}
--
-- Read wrap definitions from a JSON file
-- if a filename was given in the metadata
local function Meta (meta)
  local name = 'custom-style2latex'
  -- Normalize the style to latex variable
  if not ('table' == type(style2latex)) then
    style2latex = {
      Div = { },
      Span = { }
    }
  end
  for _, tag in ipairs(wrap_tags) do
    if not ('table' == type(style2latex[tag])) then
      style2latex[tag] = { }
    end
  end
  -- See if there is a metadata key of the name
  -- and if it has a true value
  local mval = meta[name]
  if mval then
    ifmt = " [%d]"
    if not ('List' == pdu.type(mval)) then
      mval = pdc.List({
        mval
      })
      ifmt = nil
    end
    for i, val in ipairs(mval) do
      if ifmt then
        i = ifmt:format(i)
      else
        i = ""
      end
      -- It should be a filename so make sure it is a string
      local ok, fn = pcall(pdu.stringify, val)
      if not (ok) then
        local vt = pdu.type(val)
        errorf("Couldn't stringify %s meta.%s%s: %s", vt, name, i, fn)
      end
      -- Now load JSON from the named file
      local data = load_json(fn)
      -- See if the top level of data has the keys Div and/or Span
      for _, tag in ipairs(wrap_tags) do
        do
          -- Get wrap definitions for the tag
          local defs = data[tag]
          if defs then
            if not( 'table' == type(defs)) then
              errorf("%s '%s' is not an object", fn, tag)
            end
            -- Loop thru the defs.
            -- Keys should be style names
            -- and each value the definiton for that style
            for style, def in pairs(defs) do
              if not ('string' == type(style)) then
                errorf("Expected keys in %s '%s' to be strings", fn, tag)
              end
              local def_type = pdu.type(def)
              if 'boolean' == def_type then
                if def then -- if `true`
                  -- Use the style name as the command/environment name
                  style2latex[tag][style] = str_wrap(style, tag)
                else -- if `false`
                  -- Remove the default wrap for this style if any
                  style2latex[tag][style] = nil
                end
              elseif 'string' == def_type then
                -- Assume the string is a command/environment name
                style2latex[tag][style] = str_wrap(def, tag)
              elseif 'table' == def_type then
                -- Check that the relevant keys have string or false values
                for _, k in ipairs(wrap_strings) do
                  local v = def[k]
                  if v then
                    if not ('string' == type(v)) then
                      errorf("Expected %s %s[%s].%s to be string", fn, tag, style, k)
                    end
                  end
                end
                -- Passed the test so use the def as the wrap for this style
                style2latex[tag][style] = def
              else
                errorf("Cannot use %s at %s %s[%s].%s", def_type, fn, tag, style, k)
              end
            end
          end
        end
      end
    end
  end
  return nil
end

-- Process a div
local function Div (div)
  -- Have we got a custom-style?
  local style = div.attributes['custom-style']
  if style then
    -- Have we got any markup for this style
    local wrap = style2latex.Div[style]
    if wrap then
      local rv = pdc.List({ })
      if wrap.pre then
        rv:insert(wrap.pre)
      end
      -- Convert the div to LaTeX, 
      -- stripping any leading/trailing whitespace
      rv:insert(to_latex(div, true))
      if wrap.post then
        rv:insert(wrap.post)
      end
      -- Join into a single string with 3+lines
      local latex = table.concat(rv, wrap.join or "\n")
      return raw.Block(latex)
    end
  end
  -- no match!
  return nil
end

-- Process a span
-- Much easier since we don't need to convert to LaTeX
local function Span (span)
  local style = span.attributes['custom-style']
  if style then
    local wrap = style2latex.Span[style]
    if wrap then
      local rv = pdc.List({ })
      if wrap.pre then
        -- Insert it as a raw inline
        rv:insert(raw.Inline(wrap.pre))
      end
      -- The span in the middle
      rv:insert(span)
      if wrap.post then
        -- Insert it as a raw inline
        rv:insert(raw.Inline(wrap.post))
      end
      return rv
    end
  end
  return nil
end

-- Return the filters
return {
  {
    Meta = Meta
  },
  {
    Div = Div,
    Span = Span
  }
}
